function [dsm,xdsm,ydsm] = readesrigrid(filename)
%========================= Leitura do ficheiro DSM ========================
%  Syntax: [dsm,xdsm,ydsm] = readesrigrid(filename)
%   where 
%    Dsm is a structure whith the following atributes
%     + ncols
%     + nrows
%     + cellsize
%     + grid
%     + xmin or xllcorner
%     + ymin or yllcorner
% To get the coordinates of the axis we have to do 
%    xdsm=dsm.xmin:dsm.cellsize:dsm.xmin+(dsm.ncols-1)*dsm.cellsize;
%    ydsm=dsm.ymin:dsm.cellsize:dsm.ymin+(dsm.nrows-1)*dsm.cellsize;
% This means that the pixel center i,j as the coordinates
% xci = dsm.xmin+i*cellsize/2
% ycj = dsm.ymin+j*cellsize/2

%1. Leitura do Modelo Digital de superficie
% o formato esri grid  o seguinte
% ncols             o numero de colunas (ex. 4)
% nrows             o nmero de linhas  (ex. 6)
% xllcorner        coordenada x do canto inferior esquerdo da matriz (100)
% yllcorner        coordenada y do canto inferior esquerdo da matriz (100)
% cellsize         tamanho do pixel/clula
% NODATA_value     valor dos pixeis vazios
% -9999 -9999 5 2  De seguida so colocadas as linhas da matrix de pixeis 
% -9999 20 100 36  comeando pelo canto superior esquerdo e terminado em 
% 3 8 35 10        no canto inferior direito   
% 32 42 50 6
% 88 75 27 9
% 13 5 1 -9999

% Exemplo 
% ncols         4
% nrows         6
% xllcorner     0.0
% yllcorner     25.0
% cellsize      50.0

%A leitura do ficheiro resulta na matriz matlab
%     13     5     1   NaN
%     88    75    27     9
%     32    42    50     6
%      3     8    35    10
%    NaN    20   100    36
%    NaN   NaN     5     2
% as coordenadas dos centros dos pixeis so
%       25    25    75   125
%     +------------------------>
%  50 |(1,1) (1,2) (1,3) (1,4)
%  75 |(2,1) (2,2) (2,3) (2,4)
% 125 |(3,1) (3,2) (3,3) (3,4)
% 175 |(4,1) (4,2) (4,3) (4,4)
% 225 |(5,1) (5,2) (5,3) (5,4)
% 275 |(6,1) (6,2) (6,3) (6,4)
%     |

disp('Attention: the grid is in the form BL2UR and not UL2BR')
filein=fopen(filename,'rt'); % r = read, t=text
%Ler o header
header=fscanf(filein,'%s',[1,1]);
dsm.ncols =fscanf(filein,'%i',[1,2]);
header=fscanf(filein,'%s',[1,1]);
dsm.nrows=fscanf(filein,'%i',[1,2]);
xcorner=fscanf(filein,'%s',[1,1]);
x0 = fscanf(filein,'%f',[1,2]);
ycorner=fscanf(filein,'%s',[1,1]);
y0 = fscanf(filein,'%f',[1,2]);
header=fscanf(filein,'%s',[1,1]);
dsm.cellsize = fscanf(filein,'%f',[1,2]);
header=fscanf(filein,'%s',[1,1]);
nodata = fscanf(filein,'%f',[1,1]);
% leitura da grelha. Como as celulas esto gravadas por linhas lemos em
% primeiro as colunas
dsm.grid=fscanf(filein,'%f',[dsm.ncols,dsm.nrows]);
% como lemos primeiro as colunas a grelha esta transposta
dsm.grid=dsm.grid';
%Como interessa que o primeiro elemento da matriz dsm coincida com a origem 
% das coordenadas fao
dsm.grid=flipud(dsm.grid);
%fechar o ficheiro
fclose(filein);
%calculo das coordenadas dos extremos do rectangulo que engloba o modelo
%(bounding-box)
if lower(xcorner(1:5))== 'xllco' %pode ser xllcenter
    dsm.xmin=x0;
    dsm.ymin=y0;
else %caso xllcenter
    dsm.xmin=x0-dsm.cellsize/2;
    dsm.ymin=y0-dsm.cellsize/2;
end
% As clulas sem valor atribuido (<=nodata) tomam o valor NaN
dsm.grid(dsm.grid<=nodata)=NaN;
% No caso de estramos interessados em saber as coordenadas dos nos
if nargout>1
    xdsm=dsm.xmin:dsm.cellsize:dsm.xmin+(dsm.ncols-1)*dsm.cellsize;
    ydsm=dsm.ymin:dsm.cellsize:dsm.ymin+(dsm.nrows-1)*dsm.cellsize;
end